package model

import (
	"TenantServe/app/common/model"
	"TenantServe/serve/orm"
	"fmt"
)

type SysMenu struct {
	*model.SysMenu
}

type SysMenuScene struct {
	ID uint64 `json:"id"`
	*model.SysMenuBasic
	SceneID uint64 `json:"scene_id"` // 场景（租户，角色）
}

// Recursion 递归查询子菜单
func (m *SysMenu) Recursion(id uint64) {
	//SELECT
	//t3.id, t3.parent_id, t3.`name`, t3.kind
	//FROM
	//(SELECT
	//t1.id, t1.parent_id, t1.name, t1.kind,
	//	IF(FIND_IN_SET( parent_id, @pids ) > 0, @pids := concat( @pids, ',', id ), 0 ) AS is_child
	//FROM
	//(SELECT id, parent_id, `name`, kind FROM sys_menu WHERE is_deleted = 0 ORDER BY parent_id ASC, sort DESC) AS t1,
	//	(SELECT @pids := 1) AS t2
	//) AS t3
	//WHERE is_child != 0
}

// SystemMenu 系统菜单
func (m *SysMenu) SystemMenu(where ...*model.ModelWhere) ([]*model.SysMenu, error) {
	out := make([]*model.SysMenu, 0)

	db := orm.GetDB().Table(m.TableName()).
		Where("status = ? AND is_deleted = ?", model.SysMenuStatusForNormal, model.DeleteStatusForNot)

	if len(where) > 0 {
		for _, wo := range where {
			db = db.Where(wo.Condition, wo.Value)
		}
	}
	if err := db.Order("parent_id " + model.OrderModeToAsc).Order("sort " + model.OrderModeToDesc).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

// TenantMenu 租户菜单
func (m *SysMenu) TenantMenu(tenantID uint64) ([]*SysMenuScene, error) {
	mSysTenantMenu := NewSysTenantMenu()

	out := make([]*SysMenuScene, 0)

	db := orm.GetDB().Table(m.TableName()+" m").
		Select("m.id, m.parent_id, m.name, m.kind, m.link, m.component, m.icon, t_m.id AS scene_id").
		Joins(fmt.Sprintf("LEFT JOIN %s AS t_m ON m.id = t_m.menu_id AND t_m.tenant_id = %d AND t_m.is_deleted = %d",
			mSysTenantMenu.TableName(), tenantID, model.DeleteStatusForNot)).
		Where("m.status = ? AND m.is_deleted = ?", model.SysMenuStatusForNormal, model.DeleteStatusForNot)

	if err := db.Order("m.parent_id " + model.OrderModeToAsc).Order("m.sort " + model.OrderModeToDesc).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

// RoleMenu 角色菜单
func (m *SysMenu) RoleMenu(tenantID uint64, roleID uint64) ([]*SysMenuScene, error) {
	mSysTenantMenu := NewSysTenantMenu()

	mSysRoleMenu := NewSysRoleMenu()

	out := make([]*SysMenuScene, 0)

	db := orm.GetDB().Table(m.TableName()+" AS m").
		Select("m.id, m.parent_id, m.name, m.kind, m.link, m.component, m.icon, r_m.id AS scene_id").
		Joins(fmt.Sprintf("LEFT JOIN %s AS t_m ON m.id = t_m.menu_id AND t_m.tenant_id = %d AND t_m.is_deleted = %d",
			mSysTenantMenu.TableName(), tenantID, model.DeleteStatusForNot)).
		Joins(fmt.Sprintf("LEFT JOIN %s AS r_m ON m.id = r_m.menu_id AND r_m.role_id = %d AND r_m.is_deleted = %d",
			mSysRoleMenu.TableName(), roleID, model.DeleteStatusForNot)).
		Where("m.status = ? AND m.is_deleted = ?", model.SysMenuStatusForNormal, model.DeleteStatusForNot).
		Where("t_m.id > ?", 0)

	if err := db.Order("m.parent_id " + model.OrderModeToAsc).Order("m.sort " + model.OrderModeToDesc).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

// UserMenu 用户菜单
func (m *SysMenu) UserMenu(uid uint64) ([]*SysMenuScene, error) {
	mSysUserRole := NewSysUserRole()

	mSysRoleMenu := NewSysRoleMenu()

	out := make([]*SysMenuScene, 0)

	db := orm.GetDB().Table(mSysUserRole.TableName()+" AS u_r").
		Select("m.id, m.parent_id, m.name, m.kind, m.link, m.component, m.icon").
		Joins(fmt.Sprintf("LEFT JOIN %s AS r_m ON u_r.role_id = r_m.id AND r_m.is_deletd = %d",
			mSysRoleMenu.TableName(), model.DeleteStatusForNot)).
		Joins(fmt.Sprintf("LEFT JOIN %s AS m ON r_m.menu_id = m.id AND m.status = %d AND m.is_deleted = %d",
			m.TableName(), model.SysMenuStatusForNormal, model.DeleteStatusForNot)).
		Where("u_r.uid = ? AND u_r.is_deleted = ?", uid, model.DeleteStatusForNot)

	if err := db.Order("m.parent_id " + model.OrderModeToAsc).Order("m.sort " + model.OrderModeToDesc).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

func NewSysMenu() *SysMenu {
	return &SysMenu{SysMenu: model.NewSysMenu()}
}
